﻿package carrousel3D {
	
	import flash.display.Sprite
	import flash.events.Event
	import flash.geom.Matrix
	
	import carrousel3D.XMLManager
	import carrousel3D.PhotoDisp
	import carrousel3D.Carrousel
	
	public class Main extends Sprite {
		
		private const url:String = "data.xml";	//url du XML de base contenant les url des images à charger
		private var imgTab:Vector.<PhotoDisp>;	//Tableau contenant les Sprites des images
		private var carrousel:Carrousel;		//Carrousel qui va contenir et faire tourner les images
		private var counter:int = 0;			//Compteur servant à charger les images une par une
		
		//Fonction principale déclenchant le chargement du XML
		public function Main():void {
			//drawGradientBG();													//dessine le dégradé de fond
			XMLManager.load(url);												//Lance le chargement du XML
			XMLManager.loader.addEventListener(Event.COMPLETE, loadComplete);	//Déclenché la fonction lors de la fin du chargement
		}
		
		//Fonction déclenchée après le chargement du XML
		private function loadComplete(evt:Event):void {
			imgTab = new Vector.<PhotoDisp>();										//Crée le Vector qui va contenir les images
			for (var i:int=0; i<XMLManager.imgs; i++) {								//Boucle sur toutes les photos présentes
				var photoTmp:PhotoDisp = new PhotoDisp(XMLManager.getURL(i));		//Crée un objet PhotoDisp (voir classe PhotoDisp)
				photoTmp.name = "photo"+i;											//Donne un nom à la photo
				imgTab.push(photoTmp);												//Stoque la photo dans le tableau
				if (i==0) { photoTmp.addEventListener(Event.COMPLETE, imgLoader); }	//Si la photo est la première, écoute si l'image est finie de charger
			}
			carrousel = new Carrousel(imgTab); 				//Crée le carrousel avec les images ainsi crées
			carrousel.x = stage.stageWidth/2;				//place le carrousel au centre de la scène en X
			carrousel.y = stage.stageHeight/2;				//place le carrousel au centre de la scène en Y
			carrousel.z = XMLManager.radius;				//éloigne le carrousel de la scène, permettant de mettre les images à la tailel voulue en premier plan
			addChild(carrousel)
		}
		
		//Fonction déclenchée lors de la fin de chaque chargement d'image dans un contener (PhotoDisp)
		private function imgLoader(evt:Event):void {
			counter++;
			evt.currentTarget.addEventListener("MouseThumb", showPhoto);		//Ajoute l'écouteur de clic à la souris
			evt.currentTarget.addEventListener("MouseView", hidePhoto);			//Ajoute l'écouteur de clic à la souris
			evt.currentTarget.removeEventListener(Event.COMPLETE, imgLoader);	//Détruit l'écouteur d'évènement
			if (counter<imgTab.length) {										//S'il reste des images à charger
				imgTab[counter].load();											//Lance le chargement de l'image suivante
				imgTab[counter].addEventListener(Event.COMPLETE, imgLoader);	//ajoute l'écouteur de fin de chargement
			}
		}
		
		//Fonction déclenchée lors du clic sur une Miniature
		private function showPhoto(evt:Event):void {
			carrousel.selectPhoto(evt.currentTarget as PhotoDisp);	//Lance la sélection de la photo dans le carrousel
		}
		
		//Fonction déclenchée lors du clic sur une Photo plein écran
		private function hidePhoto(evt:Event):void {
			carrousel.deselectPhoto(evt.currentTarget as PhotoDisp);	//Lance la sélection de la photo dans le carrousel
		}
		
		/*-- fonctions générales -*/
		
		//fonction dessinant le dégradé de fond
		//private function drawGradientBG():void {
			//var mat:Matrix = new Matrix();											//Matrice de déformation du dégradé
			//mat.createGradientBox(stage.stageWidth, stage.stageHeight, Math.PI/2);	//boîte de transformation de la matrice
			//graphics.beginGradientFill("linear", [0xFFFFFF,0,0x666666,0x888888,0xFFFFFF], [1,1,1,1,1], [0,128,132,146,255], mat);
			//graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);			//dessin du dégradé
		//}
		
	}
}